@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 2016-2017 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2016-2026 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 // It makes more sense to define octave_popen2 with the wrapper
 // functions than it does to try to provide wrappers for all the
@@ -41,10 +44,13 @@ along with Octave; see the file COPYING.  If not, see
 #  include <io.h>
 #  define WIN32_LEAN_AND_MEAN 1
 #  include <windows.h>
+
+#  include "uniconv-wrappers.h"
 #else
 #  include <errno.h>
 #  include <fcntl.h>
 #  include <sys/types.h>
+#  include <sys/wait.h>
 #  include <unistd.h>
 #endif
 
@@ -58,8 +64,9 @@ make_command_string (const char *cmd, char *const *args)
   char *const *argp;
   size_t cmd_len;
   char *command;
+  size_t arg_len;
 
-  // Count Command length, quotes, and terminating NUL character.
+  // Count command length, quotes, and terminating NUL character.
   cmd_len = strlen (cmd) + 3;
 
   // Count argument length, space, and quotes.
@@ -70,11 +77,22 @@ make_command_string (const char *cmd, char *const *args)
 
   command = (char *) malloc (cmd_len);
 
+  // double-quote command
   sprintf (command, "\"%s\"", cmd);
 
   argp = args;
+  cmd_len = strlen (cmd) + 2;
   while (*++argp)
-    sprintf (command, "%s \"%s\"", command, *argp);
+    {
+      // append double-quoted argument separated by space to command buffer
+      command[cmd_len++] = ' ';
+      command[cmd_len++] = '"';
+      arg_len = strlen (*argp);
+      memcpy (command + cmd_len, *argp, arg_len);
+      cmd_len += arg_len;
+      command[cmd_len++] = '"';
+    }
+  command[cmd_len] = 0;
 
   return command;
 }
@@ -134,10 +152,15 @@ octave_popen2 (const char *cmd, char *const *args, bool sync_mode,
 
   command = make_command_string (cmd, args);
 
-  status = CreateProcess (0, command, 0, 0, TRUE, 0, 0, 0, &si, &pi);
+  wchar_t *wcmd = u8_to_wchar (command);
 
   free (command);
 
+  status = CreateProcessW (NULL, wcmd, NULL, NULL, TRUE, CREATE_NO_WINDOW,
+                           NULL, NULL, &si, &pi);
+
+  free (wcmd);
+
   if (! status)
     {
       *errmsg = "popen2: process creation failed";
@@ -157,6 +180,9 @@ octave_popen2 (const char *cmd, char *const *args, bool sync_mode,
 
 #else
 
+// Magic value used to indicate child failed back to parent process
+#define OCTAVE_CHILD_FAILURE 129
+
 pid_t
 octave_popen2 (const char *cmd, char *const *args, bool sync_mode,
                int *fildes, const char **errmsg)
@@ -198,15 +224,15 @@ octave_popen2 (const char *cmd, char *const *args, bool sync_mode,
               close (child_stdout[1]);
 
               if (execvp (cmd, args) < 0)
-                perror ("popen2 (child)");
+                perror ("error: popen2 (child)");
             }
           else
-            perror ("popen2 (child)");
+            perror ("error: popen2 (child)");
         }
       else
-        perror ("popen2 (child)");
+        perror ("error: popen2 (child)");
 
-      _exit (127);
+      _Exit (OCTAVE_CHILD_FAILURE);  
     }
   else if (pid > 0)
     {
@@ -227,13 +253,36 @@ octave_popen2 (const char *cmd, char *const *args, bool sync_mode,
           fildes[0] = child_stdin[1];
           fildes[1] = child_stdout[0];
 
+          // Sanity check that child process hasn't already aborted.
+          // Note: Maybe use nanosleep() as usleep() is deprecated/removed
+          // after 2008?
+          usleep (5e3);  // wait for 5 milliseconds
+          int wstatus;
+          pid_t tst_pid = waitpid (pid, &wstatus, WNOHANG);
+          if (tst_pid == -1)
+            {
+              // Error with waitpid ()
+              *errmsg = strerror (errno);
+              return -1;
+            }
+          else if (tst_pid != 0)
+            {
+              // Child has already changed state, probably bad, but verify
+              if (WIFEXITED(wstatus)
+                  && WEXITSTATUS(wstatus) == OCTAVE_CHILD_FAILURE)
+                {
+                  *errmsg = "popen2 (parent): failure in child process";
+                  return -1;
+                }
+            }
+
           return pid;
         }
     }
 
-  *errmsg = "foobar!";
+  // Reaching this code means fork() was unsuccessful
   *errmsg = strerror (errno);
-  return pid;
+  return -1;
 }
 
 #endif
